<?php

namespace Fiscal\Logic\NotaFiscal;


use Fiscal\Entities\Rateionf;
use Fiscal\Logic\NotaFiscal\Exceptions\ImportacaoRateioException;
use Geral\Entities\Filial;
use Geral\Entities\Itemproduto;
use Geral\Util\Ambiente;
use Geral\Util\DateUtil;
use Importacao\Logic\Util\ExcelUtils;
use Indeterminado\Entities\Centcustfil;
use Setforms2\Shared\Registry;

class ImportacaoRateio
{

    const POR_PERCENTUAL = '3';
    const POR_VALOR      = '4';

    /**
     * Monta o dataset com o rateio dos produtos da nota fiscal
     * @param $rateio
     * @return array
     */
    public static function montaDatasetRateioImportado($rateio, $cdfilial, $nrlanctonf)
    {
        $itemproduto = Itemproduto::findByCdfilialAndNrlanctonf($cdfilial, $nrlanctonf);
        $soma        = self::somaValorTotalItem($itemproduto);
        return self::montaArrayRateio($rateio, $itemproduto, $soma);
    }

    /**
     * Soma os valores contabeis de todos os produtos
     * @return int
     */
    public static function somaValorTotalItem($itemproduto)
    {
        $soma = 0;
        foreach ($itemproduto as $item) {
            $soma += $item->getVrcontitem();
        }

        return $soma;
    }

    /**
     * Monta um array com os dados do arquivo importado
     * @param $itemproduto
     * @return array
     */
    public static function montaArrayRateio($rateio, $itemproduto, $soma)
    {
        if (count(reset($rateio)) == self::POR_VALOR) {
            /** @var $item Itemproduto */
            foreach ($itemproduto as $item) {
                $somaVrRateio = 0;
                foreach ($rateio as $key => $rat) {
                    if (!$rat['FILIAL']) {
                        break;
                    }
                    $dataset[$item->getNrseqitprod()][$key]['CDFILICCUST'] = $rat['FILIAL'];
                    $dataset[$item->getNrseqitprod()][$key]['CDCENTCUSNF'] = $rat['CCUSTO'];
                    $dataset[$item->getNrseqitprod()][$key]['VRRATENF']    = round((($item->getVrcontitem() * $rat['VALOR']) / $soma), 2);
                    $dataset[$item->getNrseqitprod()][$key]['DSRATEIO']    = $rat['DESCRICAO'];
                    $somaVrRateio += $dataset[$item->getNrseqitprod()][$key]['VRRATENF'];
                    $ultimo = $key;
                }
                $dif = round(($item->getVrcontitem() - $somaVrRateio), 2);

                if ($dif > 1 || $dif < -1) {
                    ImportacaoRateioException::totalRateioDifereValorContabilProduto();
                }

                $dataset[$item->getNrseqitprod()][$ultimo]['VRRATENF'] =
                    $dataset[$item->getNrseqitprod()][$ultimo]['VRRATENF'] + $dif;
            }
        } else {
            /** @var $item Itemproduto */
            foreach ($itemproduto as $item) {
                $somaVrRateio = 0;
                foreach ($rateio as $key => $rat) {
                    if (!$rat['FILIAL']) {
                        break;
                    }
                    $dataset[$item->getNrseqitprod()][$key]['CDFILICCUST'] = $rat['FILIAL'];
                    $dataset[$item->getNrseqitprod()][$key]['CDCENTCUSNF'] = $rat['CCUSTO'];
                    $dataset[$item->getNrseqitprod()][$key]['VRRATENF']    = round((($item->getVrcontitem() * $rat['PERCENTUAL']) / 100), 2);
                    $dataset[$item->getNrseqitprod()][$key]['DSRATEIO']    = '';
                    $somaVrRateio += $dataset[$item->getNrseqitprod()][$key]['VRRATENF'];
                    $ultimo = $key;
                }
                $dif                                                   = round(($item->getVrcontitem() - $somaVrRateio), 2);
                $dataset[$item->getNrseqitprod()][$ultimo]['VRRATENF'] =
                    $dataset[$item->getNrseqitprod()][$ultimo]['VRRATENF'] + round(($item->getVrcontitem() - $somaVrRateio), 2);
            }
        }
        return $dataset;
    }

    /**
     * Valida se o arquivo importado é um arquivo válido e
     * se o cabeçalho do arquivo segue o padrão
     * 'FILIAL', 'CCUSTO', 'VALOR', 'DESCRICAO', 'PERCENTUAL'
     * @param $rateio
     * @return void
     */
    public static function validaArquivoImportado($rateio)
    {
        if (!is_array($rateio) || empty($rateio)) {
            ImportacaoRateioException::arquivoInvalido();
        }

        $cabecalho = array('FILIAL', 'CCUSTO', 'VALOR', 'DESCRICAO', 'PERCENTUAL');

        foreach (reset($rateio) as $key => $t) {
            if (!in_array($key, $cabecalho)) {
                ImportacaoRateioException::arquivoInvalido();
            }
        }
    }

    /**
     * Remove o arquivo .XLS importado anteriormente
     * @param $arquivo
     * @return void
     */
    public static function removeArquivoXlsImportado($arquivo)
    {
        unlink(UPLOAD_PATH . $arquivo);
    }

    /**
     * Importa arquivo .XLS
     * @param $arquivo
     * @return array
     */
    public static function importaArquivoXls($arquivo)
    {
        $rateio = new ExcelUtils(UPLOAD_PATH . $arquivo);
        $rateio = $rateio->buildDataIntoJson();
        return $rateio;
    }

    /**
     * Valida se os centro de custo informado no arquivo .xls está cadastrado para a filial
     * @param $dataset
     * @return array|string
     */
    public static function validaCentroDeCustoFilial($dataset)
    {
        $erros = '';
        foreach (reset($dataset) as $rateio) {
            $centcustfil = Centcustfil::findOneByCdfilialAndCdcentcust($rateio['CDFILICCUST'], $rateio['CDCENTCUSNF']);
            if (!$centcustfil instanceof Centcustfil) {
                $erros[] = array('CDFILIAL' => $rateio['CDFILICCUST'], 'CDCENTCUST' => $rateio['CDCENTCUSNF']);
            }
        }
        return $erros;
    }

    /**
     * Insere os rateios importados do arquivo .xls
     * @param $rateio
     * @param $cdfilial
     * @param $nrlanctonf
     * @return void
     */
    public static function salvarImportacao($produto, $cdfilial, $nrlanctonf)
    {
        $connection = \Setforms2\Shared\Registry::getEntityManager();
        $connection->beginTransaction();
        try {
            self::deletarRateio($cdfilial, $nrlanctonf);
            Registry::getEntityManager()->flush();
            foreach ($produto as $key => $rateio) {
                foreach ($rateio as $rat) {
                    self::inserirRateio($cdfilial, $nrlanctonf, $key, $rat['CDFILICCUST'], $rat['CDCENTCUSNF'], $rat['VRRATENF'], $rat['DSRATEIO']);
                }
            }
            Registry::getEntityManager()->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            tk_abort($e->getMessage());
        }
    }

    /**
     * Deleta os rateios existentes para que os novos importados sejam inseridos
     * @param $cdfilial
     * @param $nrlanctonf
     * @return void
     */
    public static function deletarRateio($cdfilial, $nrlanctonf)
    {
        $rateios = Rateionf::findByCdfilialAndNrlanctonf($cdfilial, $nrlanctonf);
        /** @var $rateio Rateionf */
        foreach ($rateios as $rateio) {
            $rateio->delete();
        }
    }

    /**
     * Insere o rateio importado na tabela RATEIONF
     * @param      $cdfilial
     * @param      $nrlanctonf
     * @param      $nrseqitprod
     * @param      $cdfiliccust
     * @param      $cdcentcusnf
     * @param      $vrratenf
     * @param null $dsrateio
     * @return void
     */
    public static function inserirRateio($cdfilial, $nrlanctonf, $nrseqitprod, $cdfiliccust, $cdcentcusnf, $vrratenf, $dsrateio = null)
    {
        $rateionf = new Rateionf();

        $rateionf->setNrorg(Ambiente::getNrorg());
        $rateionf->setCdfilial($cdfilial);
        $rateionf->setNrlanctonf($nrlanctonf);
        $rateionf->setNrseqitprod($nrseqitprod);
        $rateionf->setCdfiliccust($cdfiliccust);
        $rateionf->setCdcentcusnf($cdcentcusnf);
        $rateionf->setVrratenf($vrratenf);
        $rateionf->setDsrateio($dsrateio);

        $rateionf->setDtinclusao(DateUtil::getDataAtual());
        $rateionf->setDtultatu(DateUtil::getDataAtual());
        $rateionf->setNrorginclusao(Ambiente::getNrorg());
        $rateionf->setNrorgultatu(Ambiente::getNrorg());
        $rateionf->setCdoperinclusao(Ambiente::getCdOperador());
        $rateionf->setCdoperultatu(Ambiente::getCdOperador());
        $rateionf->setIdativo('S');

        $rateionf->persist();
    }
}